import os
import os.path as osp

from cubicweb.cwctl import CWCTL

from cubes.dataio.xy import XY
from cubes.dataio import interfaces
from cubes.dataio.dataimport import MassiveObjectStore, RDFStore
from cubes.dataio.ccplugin import ImportRDFCommand, _init_cw_connection


class DatabnfImportRDFCommand(ImportRDFCommand):
    """
    Command for importing databnf rdf data
    """
    name = 'import-databnf-rdf'

    def _create_store(self, session, XY):
        external_uris = dict(session.execute('Any U, X WHERE X is ExternalUri, X uri U'))
        internal_store = MassiveObjectStore(session, replace_sep=u' ',
                                            eids_seq_range=10000,
                                            commit_at_flush=False,
                                            autoflush_metadata=False)
        store = RDFStore(session, XY,
                         internal_store=internal_store,
                         external_uris_dict=external_uris)
        self.internal_count = 0
        return store, internal_store

    def _step_flush(self, store, internal_store):
        if self.internal_count and self.internal_count % 10 == 0:
            print 'Flushing', self.internal_count
            internal_store.flush()
        self.internal_count += 1

    def _final_flush(self, store, internal_store):
        internal_store.flush()
        internal_store.flush_meta_data()
        store.convert_all_relations()
        internal_store.cleanup()
        store.commit()
        internal_store.commit()

    def cleanup(self, session):
        # Remove "empty" entity relations
        for subj, obj, new_rtype, etype in (('manifestation', 'work',
                                             'work_manifested', 'WorkManifested'),
                                            ('manifestation', 'contributor',
                                             'contributors', 'Contribution'),
                                            ('manifestation', 'subject',
                                             'subject_manifested', 'SubjectManifested')):
            # Delete entity table
            _sql = ('DELETE FROM cw_%s AS C WHERE NOT EXISTS ('
                    'SELECT * FROM %s_relation AS SU WHERE C.cw_eid=SU.eid_from) '
                    'OR NOT EXISTS (SELECT * FROM %s_relation AS SU WHERE C.cw_eid=SU.eid_from)'
                    % (etype.lower(), subj, obj))
            session.system_sql(_sql)
            # Delete from entities
            _sql = ("DELETE FROM entities AS C WHERE C.type='%s' "
                    "AND NOT EXISTS(SELECT * FROM cw_%s AS T WHERE T.cw_eid=C.eid)"
                    % (etype, etype.lower()))
            session.system_sql(_sql)
            # Cleanup meta tables
            for table in ('is_relation', 'is_instance_of_relation',
                          'cw_source_relation', 'owned_by_relation', 'created_by_relation',
                          '%s_relation' % subj, '%s_relation' % obj):
                _sql = ("DELETE FROM %s AS C WHERE NOT EXISTS ( "
                        "SELECT E.eid FROM entities AS E WHERE C.eid_from=E.eid)" % table)
                session.system_sql(_sql)
            # Inserting shortcuts
            _sql = ('INSERT INTO %(r)s_relation SELECT DISTINCT SU.eid_to, OB.eid_to '
                    'FROM %(s)s_relation AS SU, %(o)s_relation AS OB '
                    'WHERE SU.eid_from=OB.eid_from'
                    % {'r': new_rtype, 's': subj, 'o': obj})
            session.system_sql(_sql)
        session.commit()
        session.set_cnxset()

    def print_stats(self, session):
        # Print some statistics
        etypes = ('Work', 'Expression', 'Manifestation', 'Item',
                  'Contribution', 'WorkManifested', 'SubjectManifested',
                  'Agent', 'Person', 'CorporateBody', 'Family',
                  'Place',' Event', 'Name', 'Subject', 'Concept', 'Object')
        for etype in etypes:
            print etype, '>>>', session.execute('Any COUNT(X) WHERE X is %s' % etype)[0][0]
        rtypes = ('realized_through', 'embodied_in', 'exemplified_by',
                  'main_author', 'close_match', 'exact_match',
                  'same_as', 'see_also', 'depictions',
                  'work_manifested', 'contributors', 'subject_manifested',
                  'related_concept', 'broader_concept', 'aggregated_by')
        for rtype in rtypes:
            print rtype, '>>>', session.execute('Any COUNT(X) WHERE X %s Y' % rtype)[0][0]


    def group_files(self, directory):
        """ Create couples of files that should be imported together
        """
        # We have a list from dataio command
        directory = directory[0]
        seen_files = set()
        for fname in os.listdir(directory):
            fname = osp.join(directory, fname)
            if fname in seen_files:
                continue
            seen_files.add(fname)
            # Compute related file fname
            try:
                num = osp.splitext(fname)[0].rsplit('_', 1)[-1]
                next_num = str('%04i' % (int(num) + 1))
            except:
                continue
            if 'editions' in fname and '__manif_' in fname:
                _new_fname = fname.replace('__manif_%s' % num,
                                           '__expr_%s' % next_num)
                if osp.exists(_new_fname):
                    seen_files.add(_new_fname)
                    yield (fname, _new_fname)
                    continue
            elif 'person_authors__skos_' in fname:
                _new_fname = fname.replace('_person_authors__skos_%s' % num,
                                           '_person_authors__foaf_%s' % next_num)
                if osp.exists(_new_fname):
                    seen_files.add(_new_fname)
                    yield (fname, _new_fname)
                    continue
            elif 'org_authors__skos_' in fname:
                _new_fname = fname.replace('_org_authors__skos_%s' % num,
                                           '_org_authors__foaf_%s' % next_num)
                if osp.exists(_new_fname):
                    seen_files.add(_new_fname)
                    yield (fname, _new_fname)
                    continue
            elif '_works__skos_' in fname:
                _new_fname = fname.replace('_works__skos_%s' % num,
                                           '_works__frbr_%s' % next_num)
                if osp.exists(_new_fname):
                    seen_files.add(_new_fname)
                    yield (fname, _new_fname)
                    continue
            elif '_rameau__skos_' in  fname:
                yield (fname,)
                continue
            elif ('_works__manifestations_' in fname or '_contributions_' in fname
                  or '_docrameau_' in fname or '_study_' in fname):
                yield (fname,)
                continue

    def run(self, args):
        appid = args.pop(0)
        cw_cnx, session = _init_cw_connection(appid)
        session.set_pool()
        # Create the internal store and the RDF store
        store, internal_store = self._create_store(session, XY)
        # Push grouped filenames
        for _filenames in self.group_files(args):
            if len(_filenames) == 1 and 'rameau__skos' in _filenames[0] :
                dicts= interfaces.iterate_build_uri_dict(_filenames, library=self.config.lib,
                                                         rdf_format=self.config.rdf_format,
                                                         max_size=1000)
            else:
                uri_dictionary = interfaces.build_uri_dict(_filenames, library=self.config.lib,
                                                           rdf_format=self.config.rdf_format)
                dicts = (uri_dictionary,)
            for uri_dictionary in dicts:
                uri_dictionary = XY.merge_uri_dictionary(uri_dictionary)
                # Import entities
                self.push_uri_dictionary(store, uri_dictionary)
                # Flush
                self._step_flush(store, internal_store)
        # Final flush
        print 'Final flush'
        self._final_flush(store, internal_store)
        print 'Cleanup'
        self.cleanup(session)
        self.print_stats(session)


CWCTL.register(DatabnfImportRDFCommand)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-frbr schema"""

# See http://archive.ifla.org/VII/s13/frbr/frbr2.htm

from yams.buildobjs import (EntityType, RelationDefinition, SubjectRelation, Boolean, Float,
                            String, Int, BigInt, Date)
from yams.constraints import SizeConstraint
from cubicweb.schemas.base import ExternalUri

addr_constraints = ExternalUri.get_relation('uri').constraints
for cstr in addr_constraints[:]:
    if isinstance(cstr, SizeConstraint):
        addr_constraints.remove(cstr)
        break


class _AbstractEntity(EntityType):
    __abstract__ = True
    title = String(required=True, indexed=True, fulltextindexed=True)
    frbnf = Int(indexed=True)
    preferred_label =SubjectRelation('Name', cardinality='1?', inlined=True)
    alt_labels =SubjectRelation('Name', cardinality='**')
    deweys =SubjectRelation('Dewey', cardinality='**')
    description = String(fulltextindexed=True)
    id = Int(indexed=True) # Other possible ids, e.g. mb_id
    type = String(maxsize=256, indexed=True) # E.g. for corporate bodies, places, ...


### WORK ELEMENTS
class Work(_AbstractEntity):
    start_date = Date()
    stop_date = Date()
    musical_genre = SubjectRelation('ExternalUri', cardinality='**')
    nationality = SubjectRelation('Country', cardinality='?*')
    languages = SubjectRelation('Language', cardinality='**')


class Expression(EntityType):
    pass


class Manifestation(EntityType):
    title = String(maxsize=2048, indexed=True, fulltextindexed=True)
    ean = String(maxsize=256, indexed=True)
    ismn = String(maxsize=256, indexed=True)
    isbn = String(maxsize=256, indexed=True)
    frbnf = Int(indexed=True)
    formatted_isbn = BigInt(indexed=True)
    dcmitype = String(maxsize=64, indexed=True)
    edition_date = Date()
    edition = String()
    publisher = String()
    publication = String()
    publication_place = String()
    description = String(fulltextindexed=True)
    languages = SubjectRelation('Language', cardinality='**')
    deweys =SubjectRelation('Dewey', cardinality='**')
    numerized_editions = SubjectRelation('ExternalUri', cardinality='*?')
    publication_location = SubjectRelation('Place', cardinality='?*', inlined=True)



class Item(EntityType):
    number = BigInt(indexed=True)
    cote = String(maxsize=64)
    section = String(maxsize=64)
    location = SubjectRelation('Place', cardinality='?*', inlined=True)


### CONTRIBUTION ELEMENTS
class Contribution(EntityType):
    # TODO: Not inlined for now as inlined relation conversion is
    # not possible in the current MassiveObjectStore
    manifestation = SubjectRelation(('Expression', 'Manifestation'), cardinality='1*')
    contributor = SubjectRelation(('Agent', 'Person', 'CorporateBody', 'Family'), cardinality='1*')
    role = SubjectRelation('Role', inlined=True, cardinality='1*')


class WorkManifested(EntityType):
    # TODO: Not inlined for now as inlined relation conversion is
    # not possible in the current MassiveObjectStore
    manifestation = SubjectRelation(('Expression', 'Manifestation'), cardinality='1*')
    work = SubjectRelation('Work', cardinality='1*')
    auto_create = Boolean(default=False)


class SubjectManifested(EntityType):
    # TODO: Not inlined for now as inlined relation conversion is
    # not possible in the current MassiveObjectStore
    manifestation = SubjectRelation(('Expression', 'Manifestation'), cardinality='1*')
    subject = SubjectRelation(('Subject', 'Concept', 'Place', 'Event', 'Object',
                               'Work', 'Agent', 'Person', 'CorporateBody', 'Family'),
                              cardinality='1*')
    auto_create = Boolean(default=False)


### AUTHORITIES ELEMENTS
class Agent(_AbstractEntity):
    """ http://rdvocab.info/uri/schema/FRBRentitiesRDA/Agent """
    ipi = String(maxsize=256, indexed=True)
    isni = String(maxsize=256, indexed=True)
    nationality = SubjectRelation('Country', cardinality='?*')
    languages = SubjectRelation('Language', cardinality='**')


class Person(Agent):
    """ http://rdvocab.info/uri/schema/FRBRentitiesRDA/Person"""
    given_name = String(maxsize=256, indexed=True, fulltextindexed=True)
    family_name = String(maxsize=256, indexed=True, fulltextindexed=True)
    dates = String(maxsize=256, indexed=True, fulltextindexed=True)
    gender = String(vocabulary=(_('male'), _('female')))
    birthdate = Date()
    deathdate = Date()
    # Link to Place ?
    birthplace = String(maxsize=256, indexed=True, fulltextindexed=True)
    deathplace = String(maxsize=256, indexed=True, fulltextindexed=True)
    birth_location = SubjectRelation('Place', cardinality='?*', inlined=True)
    death_location = SubjectRelation('Place', cardinality='?*', inlined=True)


class CorporateBody(Agent):
    start_date = Date()
    stop_date = Date()
    activity_start = Date()
    activity_stop = Date()
    # Link to Place ?
    start_place = String(maxsize=256, indexed=True, fulltextindexed=True)
    stop_place = String(maxsize=256, indexed=True, fulltextindexed=True)
    start_location = SubjectRelation('Place', cardinality='?*', inlined=True)
    stop_location = SubjectRelation('Place', cardinality='?*', inlined=True)


class Family(Agent):
    family_name = String(maxsize=256, indexed=True, fulltextindexed=True)
    start_location = SubjectRelation('Place', cardinality='?*', inlined=True)
    stop_location = SubjectRelation('Place', cardinality='?*', inlined=True)


### SUBJECT ELEMENTS
class Subject(_AbstractEntity):
    pass


class Concept(Subject):
    pass


class Place(Subject):
    latitude = Float(required=True)
    longitude = Float(required=True)
    country = SubjectRelation('Country', cardinality='?*', inlined=True)


class Event(Subject):
    start_date = Date()
    end_date = Date()


class Object(Subject):
    pass


### SECONDARY ELEMENTS
class Name(EntityType):
    title = String(indexed=True, fulltextindexed=True)
    id = Int(indexed=True) # Other possible ids, e.g. mb_id
    type = String(maxsize=256, indexed=True)
    start_date = Date()
    stop_date = Date()
    additional_infos = String(maxsize=1024, indexed=True, fulltextindexed=True)


### VOCABULARIES
class _AbstractVocabulary(EntityType):
    __abstract__ = True
    name = String(maxsize=256, internationalizable=True)
    code = String(unique=True, maxsize=10)
    frequency = Int(indexed=True)
    sort_key = Int(unique=True, indexed=True)
    id = Int(indexed=True) # Other possible ids, e.g. mb_id


class Country(_AbstractVocabulary):
    altlabels = String(maxsize=512) # joined with '###'


class Language(_AbstractVocabulary):
    code2 = String(maxsize=3, description='alternative code (XXX)')
    code3 = String(maxsize=2, description='2-letters code')


class Script(_AbstractVocabulary):
    iso_number = Int(indexed=True)
    language = SubjectRelation('Language', cardinality='?*', inlined=True)


class Dewey(_AbstractVocabulary):
    pass


class Role(_AbstractVocabulary):
    idloc = String(maxsize=64, indexed=True)


### RELATIONS
class realized_through(RelationDefinition):
    subject = 'Work'
    object = 'Expression'
    cardinality = '**'

class embodied_in(RelationDefinition):
    subject = 'Expression'
    object = 'Manifestation'
    cardinality = '**'

class exemplified_by(RelationDefinition):
    subject = 'Manifestation'
    object = 'Item'
    cardinality = '**'

class main_author(RelationDefinition):
    subject = 'Work'
    object = ('Agent', 'Person', 'CorporateBody', 'Family')
    cardinality = '**'

class close_match(RelationDefinition):
    subject = '*'
    object = 'ExternalUri'
    cardinality = '**'

class exact_match(RelationDefinition):
    subject = '*'
    object = 'ExternalUri'
    cardinality = '**'

class same_as(RelationDefinition):
    subject = '*'
    object = 'ExternalUri'
    cardinality = '**'

class see_also(RelationDefinition):
    subject = '*'
    object = 'ExternalUri'
    cardinality = '**'

class depictions(RelationDefinition):
    subject = '*'
    object = 'ExternalUri'
    cardinality = '**'

class related_concept(RelationDefinition):
    subject = 'Concept'
    object = 'Concept'
    cardinality = '**'

class broader_concept(RelationDefinition):
    subject = 'Concept'
    object = 'Concept'
    cardinality = '**'

class aggregated_by(RelationDefinition):
    subject = 'Work'
    object = 'Work'
    cardinality = '?*'

# Shortcut
class work_manifested(RelationDefinition):
    subject = 'Manifestation'
    object = 'Work'
    cardinality = '**'

class contributors(RelationDefinition):
    subject = 'Manifestation'
    object = ('Agent', 'Person', 'CorporateBody', 'Family')
    cardinality = '**'

class subject_manifested(RelationDefinition):
    subject = 'Manifestation'
    object = ('Subject', 'Concept', 'Place', 'Event', 'Object',
              'Work', 'Agent', 'Person', 'CorporateBody', 'Family')
    cardinality = '**'



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
""" Conversion rules for RDF """
from datetime import datetime

from cubes.dataio import xy


# UTILITY FUNCTIONS
def normalize_uri(uri):
    return uri.split('#', 1)[0]


def _convert_date(_object):
    """ Convert an rdf value to a date
    """
    try:
        return datetime.strptime(_object, '%d-%m-%Y')
    except:
        date = '01-01-' + _object.replace('.', '0')
        try:
            return datetime.strptime(date, '%d-%m-%Y')
        except:
            return None


def _specific_etype_callback(rdf_properties, types):
    """ Correctly retrieve the Concept etype
    """
    if 'skos:prefLabel' in rdf_properties and 'owl:sameAs' in rdf_properties:
        for url, _type in rdf_properties['owl:sameAs']:
            # No type for Rameau, use link to Stitch
            if url.startswith('http://stitch.cs.vu.nl/vocabularies'):
                return 'Concept'
    if 'foaf:Person' in types:
        name = rdf_properties.get('skos:prefLabel')
        if name and '(famille)' in name[0][0]:
            return 'Family'
    elif not types and ('dc:contributor' in rdf_properties and
                        len([r for r in rdf_properties
                             if r.startswith('bnfroles')])):
        # At least one roles
        return 'Contribution'
    elif not types and 'rdarelationships:workManifested' in rdf_properties:
        # At least one roles
        return 'WorkManifested'
    elif not types and 'dc:subject' in rdf_properties:
        # At least one roles
        return 'SubjectManifested'


def _build_name_callback(name, rdf_properties):
    """ Build a Name entity
    """
    return 'Name', {'title': name}


def format_isbn(rdf_properties):
    if 'bnf-onto:isbn' in rdf_properties:
        isbn = rdf_properties['bnf-onto:isbn'][0]
        isbn = [i for i in isbn if i in '0123456789']
        return int(''.join(isbn)) if isbn else None


def format_contribution_cwuri(rdf_properties):
    author = normalize_uri(rdf_properties['dc:contributor'][0][0]).split('/')[-1]
    manif = normalize_uri(rdf_properties['rdf:about'][0][0]).split('/')[-1]
    role = [r for r in rdf_properties if r.startswith('bnfroles')][0]
    role = role.split(':')[-1]
    cwuri = u'http://data.bnf.fr/contribution/%s/%s/%s' % (author, manif, role)
    return cwuri


def format_workmanifested_cwuri(rdf_properties):
    manif = normalize_uri(rdf_properties['rdarelationships:workManifested'][0][0]).split('/')[-1]
    work = normalize_uri(rdf_properties['rdf:about'][0][0]).split('/')[-1]
    cwuri = u'http://data.bnf.fr/workmanifested/%s/%s' % (work, manif)
    return cwuri


def format_subjectmanifested_cwuri(rdf_properties):
    subject = normalize_uri(rdf_properties['dc:subject'][0][0]).split('/')[-1]
    manif = normalize_uri(rdf_properties['rdf:about'][0][0]).split('/')[-1]
    cwuri = u'http://data.bnf.fr/subjectmanifested/%s/%s' % (subject, manif)
    return cwuri


def format_contribution_manifestation(rel, predicate, rdf_properties):
    return normalize_uri(rdf_properties['rdf:about'][0][0])


def format_workmanifested_work(rel, predicate, rdf_properties):
    return normalize_uri(rdf_properties['rdarelationships:workManifested'][0][0])


def format_subjectmanifested_subject(rel, predicate, rdf_properties):
    return normalize_uri(rdf_properties['dc:subject'][0][0])


def format_contribution_role(rel, predicate, rdf_properties):
    code = predicate.split(':')[-1]
    return u'http://data.bnf.fr/vocabulary/roles/%s' % code


# INITIALISATION
xy.register_prefix('marcrel', 'http://id.loc.gov/vocabulary/relators/')
xy.register_prefix('bnfroles', 'http://data.bnf.fr/vocabulary/roles/')
xy.register_prefix('databnf', 'http://data.bnf.fr/ontologie/')
xy.register_prefix('bnf-onto', 'http://data.bnf.fr/ontology/bnf-onto/')
xy.register_prefix('ark', 'http://ark.bnf.fr/ark:/12148/')
xy.register_prefix('mo', 'http://musicontology.com/')
xy.register_prefix('ore', 'http://www.openarchives.org/ore/terms/')

xy.set_base_uri(u'http://data.bnf.fr')
xy.set_uri_conversion_callback(normalize_uri)
xy.register_attribute_callback('Date', _convert_date)


# ETYPES
xy.add_equivalence('Manifestation', 'frbr:Manifestation')
xy.add_equivalence('Work', 'frbr:Work')
xy.add_equivalence('Person', 'foaf:Person')
xy.add_equivalence('CorporateBody', 'foaf:Organization')
xy.set_etype_callback(_specific_etype_callback)

xy.add_equivalence('* frbnf', 'bnf-onto:FRBNF')
xy.add_equivalence('* title', 'skos:prefLabel')
xy.add_equivalence('* title', 'rdfs:label')
xy.add_equivalence('* preferred_label Name', 'rdfs:label')
xy.add_equivalence('* preferred_label Name', 'skos:prefLabel')
xy.register_relations_etype_callbacks('preferred_label', _build_name_callback)
xy.add_equivalence('* alt_labels Name', 'skos:altLabel')
xy.register_relations_etype_callbacks('alt_labels', _build_name_callback)
xy.add_equivalence('* description', 'dc:description')
xy.add_equivalence('* description', 'skos:scopeNote')
xy.add_equivalence('* deweys Dewey', 'rdagroup2elements:fieldOfActivityOfThePerson')
xy.add_equivalence('* languages Language', 'rdagroup2elements:languageOfThePerson')
xy.add_equivalence('* nationality Country', 'rdagroup2elements:countryAssociatedWithThePerson')
xy.add_equivalence('* depictions ExternalUri', 'foaf:depiction')
xy.add_equivalence('* depictions ExternalUri', 'dbpediaowl:thumbnail')
xy.add_equivalence('* description', 'dbpediaowl:abstract')
xy.add_equivalence('* same_as ExternalUri', 'owl:sameAs')
xy.add_equivalence('* see_also ExternalUri', 'foaf:page')
xy.add_equivalence('* see_also ExternalUri', 'rdfs:seeAlso')
xy.add_equivalence('* see_also ExternalUri', 'foaf:isPrimaryTopicOf')
xy.add_equivalence('* close_match ExternalUri', 'skos:closeMatch')
xy.add_equivalence('* exact_match ExternalUri', 'skos:exactMatch')


xy.set_vocabulary('Country')
xy.set_vocabulary('Language')
xy.set_vocabulary('Script')
xy.set_vocabulary('Dewey')
xy.set_vocabulary('Role')


# AGENTS
xy.add_equivalence('Person given_name', 'foaf:givenName')
xy.add_equivalence('Person family_name', 'foaf:familyName')
xy.add_equivalence('Person dates', 'dc:date')
xy.add_equivalence('Person gender', 'foaf:gender')
xy.add_equivalence('Person birthplace', 'rdagroup2elements:placeOfBirth')
xy.add_equivalence('Person birthdate', 'rdagroup2elements:dateOfBirth')
xy.add_equivalence('Person deathplace', 'rdagroup2elements:placeOfDeath')
xy.add_equivalence('Person deathdate', 'rdagroup2elements:dateOfDeath')

xy.add_equivalence('CorporateBody start_date', 'dc:date')
xy.add_equivalence('CorporateBody start_date', 'rdagroup2elements:dateAssociatedWithTheCorporateBody')
xy.add_equivalence('CorporateBody start_place', 'rdagroup2elements:placeAssociatedWithTheCorporateBody')
xy.add_equivalence('CorporateBody activity_date', 'rdagroup2elements:dateOfEstablishmet')
xy.add_equivalence('CorporateBody description', 'rdagroup2elements:corporateHistory')
xy.add_equivalence('CorporateBody deweys Dewey', 'rdagroup2elements:fieldOfActivityOfTheCorporateBody')


# WORK/EXPRESSION/MANIFESTATION
xy.add_equivalence('Work start_date', 'dc:date')
xy.add_equivalence('Work aggregated_by Work', 'ore:isAggregatedBy')
xy.add_equivalence('Work musical_genre ExternalUri', 'mo:genre')
xy.add_equivalence('Work main_author *', 'dc:creator')
xy.add_equivalence('Work main_author *', 'dc:creator')

xy.add_equivalence('Manifestation title', 'dc:title')
xy.register_attribute_callback('Manifestation formatted_isbn', format_isbn)
xy.add_equivalence('Manifestation isbn', 'bnf-onto:isbn')
xy.add_equivalence('Manifestation ean', 'bnf-onto:EAN')
xy.add_equivalence('Manifestation ismn', 'bnf-onto:ISMN')
xy.add_equivalence('Manifestation edition', 'rdagroup1elements:designationOfEdition')
xy.add_equivalence('Manifestation edition_date', 'dc:date')
xy.add_equivalence('Manifestation publisher', 'rdagroup1elements:publishersName')
xy.add_equivalence('Manifestation publication_place', 'rdagroup1elements:placeOfPublication')
xy.add_equivalence('Manifestation publication', 'dc:publisher')
xy.add_equivalence('Manifestation dcmitype', 'dc:type')
xy.add_equivalence('Manifestation numerized_editions ExternalUri', 'rdarelationships:electronicReproduction')
xy.add_equivalence('Manifestation languages Language', 'dc:language')


# SUBJECT/CONCEPT
xy.add_equivalence('Concept broader_concept Concept', 'skos:broader')
xy.add_equivalence('Concept related_concept Concept', 'skos:related')


# CONTRIBUTION AND OTHER ENTITY-RELATIONS
xy.register_attribute_callback('Contribution cwuri',
                               format_contribution_cwuri)
xy.add_equivalence('Contribution manifestation *', 'rdf:about')
xy.register_relation_callback('Contribution manifestation *',
                              format_contribution_manifestation)
xy.add_equivalence('Contribution contributor *', 'dc:contributor')
xy.add_equivalence('Contribution role Role', 'bnfroles:*')
xy.register_relation_callback('Contribution role Role',
                              format_contribution_role)

xy.register_attribute_callback('WorkManifested cwuri',
                               format_workmanifested_cwuri)
xy.add_equivalence('WorkManifested manifestation *', 'rdf:about')
xy.register_relation_callback('WorkManifested manifestation *',
                              format_contribution_manifestation)
xy.add_equivalence('WorkManifested work *', 'rdarelationships:workManifested')
xy.register_relation_callback('WorkManifested work *',
                              format_workmanifested_work)

xy.register_attribute_callback('SubjectManifested cwuri',
                               format_subjectmanifested_cwuri)
xy.add_equivalence('SubjectManifested manifestation *', 'rdf:about')
xy.register_relation_callback('SubjectManifested manifestation *',
                              format_contribution_manifestation)
xy.add_equivalence('SubjectManifested subject *', 'dc:subject')
xy.register_relation_callback('SubjectManifested subject *',
                              format_subjectmanifested_subject)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-frbr postcreate script, executed at instance creation time or when
the cube is added to an existing instance.

You could setup site properties or a workflow here for example.
"""
import os.path as osp
import codecs

from cubicweb import dataimport as cwdi


def _import_langs(session, lang_file):
    """ Import a language file """
    with codecs.open(lang_file, encoding='utf-8') as lines:
        for line in lines:
            row = line.strip().split('|')
            if len(row[0]) > 3:
                row[0] = row[0].split('-')[0]
            code1, code2, code3, en_label, fr_label = row
            session.create_entity('Language', code=code1, code2=code2, code3=code3, name=fr_label,
                                  cwuri=u'http://id.loc.gov/vocabulary/iso639-2/%s' % code1)

def _import_countries(session, country_file):
    """ Import a country file """
    for row in cwdi.ucsvreader(open(country_file), encoding='utf-8'):
        if len(row) == 2:
            code, label = row
            altlabels = u''
        else:
            code, label, altlabels = row
        session.create_entity('Country', code=code, name=label, altlabels=altlabels,
                              cwuri='http://id.loc.gov/vocabulary/countries/%s' % code)

def _import_dewey(session, dewey_file):
    """ Import a dewey file """
    for row in cwdi.ucsvreader(open(dewey_file), encoding='utf-8'):
        code, label = row[0].strip(), row[1].strip()
        session.create_entity('Dewey', code=code, name=label,
                              cwuri=u'http://dewey.info/class/%s/' % code)

def _import_scripts(session, script_file):
    """ Import a script file - From Musicbrainz """
    for row in cwdi.ucsvreader(open(script_file), separator='\t', encoding='utf-8'):
        try:
            frequency = int(row[4])
        except ValueError:
            frequency = None
        session.create_entity('Script', code=row[1], iso_number=int(row[2]),
                              name=row[3], frequency=frequency)

def _import_role(session, role_file, idloc_file):
    """ Import a role file"""
    code_idloc = dict([code, idloc] for code, idloc in cwdi.ucsvreader(open(idloc_file), separator=','))
    for row in cwdi.ucsvreader(open(role_file), encoding='utf-8'):
        code, label = row
        sort_key = 10000*int(code[-1]) + int(code)
        if code == '0070':
            sort_key = 100001
        if code == '9990':
            sort_key = 100002
            label = u'Autre'
        try:
            cwuri = u'http://data.bnf.fr/vocabulary/roles/r%s' % int(code)
        except:
            continue
        session.create_entity('Role', cwuri=cwuri, code=code, name=label,
                              sort_key=sort_key, idloc=code_idloc.get(code))

HERE = osp.join(osp.abspath(osp.dirname(__file__)))
_import_langs(session, osp.join(HERE, 'ISO-639-2_utf-8.txt'))
_import_countries(session, osp.join(HERE, 'ISO-3166-alpha2_utf-8.txt'))
_import_dewey(session, osp.join(HERE, 'ref_domaines.csv'))
_import_scripts(session, osp.join(HERE, 'scripts.txt'))
_import_role(session, osp.join(HERE, 'ref_roles.csv'), osp.join(HERE, 'codes_marc_idlocgov.csv'))
